#include <stdio.h>
#include "htc.h"						// HI-TECH C
#include "lcd.h"						// Character LCD support

//
//	Config
//
//	CPU Configuration Fuses bits
__CONFIG(0x3fff

// Protection of program code 
//& PROTECT
& UNPROTECT

// CCP1 Pin selection 
& CCPRB0
//& CCPRB3

// In-Circuit Debugger Mode 
//& DEBUGEN
& DEBUGDIS

// Flash Program Memory Write Enable 
& UNPROTECT
//& WP0
//& WP1
//& WPA

// Data EE Memory Code Protection 
& UNPROTECT
//& CPD

// Low Voltage Programming Enable 
//& LVPEN
& LVPDIS

// Brown out detection enable 
//& BOREN
& BORDIS

// Master clear reset pin function
& MCLREN
//& MCLRDIS

// Power up timer enable 
& PWRTDIS
//& PWRTEN

// Watchdog timer enable 
//& WDTEN
& WDTDIS

// Oscillator configurations 
//& RCCLK
//& RCIO
//& INTCLK
& INTIO
//& EC
//& HS
//& XT
//& LP
);

//
// Connect stdio -> LCD
//
void putch(char c) {
	LCD_WriteData(c);
}

//
// Read A/D channel
//
unsigned short getad(unsigned char ch) {
	ADCON0 = (ADCON0 & 0b11000111) | (ch << 3);
	ADIF = 0;
	ADGO = 1;
	while(!ADIF);
	return((ADRESH << 8) | ADRESL);
}
unsigned short getads(unsigned char ch, unsigned char samp) {
	long s;
	unsigned char i;

	s = 0;
	for(i = samp; i > 0; --i) {
		s += getad(ch);
		__delay_ms(1);
	}
	return(s / samp);
}

//
// Sound
//
void beep(void){
	signed char	i;

	for(i = 100; i > 0; --i) {
		PORTB ^= 0b10000000;
		__delay_ms(1);
	}
}
void click(void){
	signed char	i;

	for(i = 5; i > 0; --i) {
		PORTB ^= 0b10000000;
		__delay_ms(1);
	}
}

//
//  M a i n
//
main(){
	unsigned char	adch;
	short	ofs_i1 = 0;
	short	ofs_i2 = 0;
	short	ofs_i3 = 0;
	unsigned char	sw_a = 0;
	unsigned char	sw_b = 0;
	unsigned char	sw_c = 0;

//
//		*** System clock ***
//
//				  OSCCON - Oscillator control register
//				  -0000000  Initial
//				  .XXX....  IRCF: Internal oscillator
//				  .000....     31.25KHz
//				  .001....     125KHz
//				  .010....     250KHz
//				  .011....     500KHz
//				  .100....     1MHz
//				  .101....     2MHz
//				  .110....     4MHz
//				  .111....     8MHz
//				  ....X...  OSTS: Oscillator Start-up Time-out Status bit
//				  ....0...     Device is running from T1OSC or INTRC as secondary system clock
//				  ....1...     Device is running from the primery system clock
//				  .....X..  IOFS:INTOSC Frequency stable bit
//				  .....1..     stable
//				  .....0..     not stable
//				  ......XX  SCS: Oscillator Mode Select bits
//				  ......00    Oscillator mode defined by FOSC<2:0>
//				  ......01    T1OSC is used by system clock
//				  ......10    Internal RC is used for system clock
//				  ......11    Reserved
#if _XTAL_FREQ == 31250					//
	OSCCON =    0b00000000;				// Internal 31.25KHz
#elif _XTAL_FREQ == 125000				//
	OSCCON =    0b00010000;				// Internal 125KHz
#elif _XTAL_FREQ == 250000				//
	OSCCON =    0b00100000;				// Internal 250KHz
#elif _XTAL_FREQ == 500000				//
	OSCCON =    0b00110000;				// Internal 500KHz
#elif _XTAL_FREQ == 1000000				//
	OSCCON =    0b01000000;				// Internal 1MHz
#elif _XTAL_FREQ == 2000000				//
	OSCCON =    0b01010000;				// Internal 2MHz
#elif _XTAL_FREQ == 4000000				//
	OSCCON =    0b01100000;				// Internal 4MHz
#elif _XTAL_FREQ == 8000000				//
	OSCCON =    0b01110000;				// Internal 8MHz
#else									//
#error "_XTAL_FREQ value"				//
#endif									//
//				  OSCTUNE - Oscillator tuning register
//				  00000000  Initial
//				  ..XXXXXX  TUN: Frequency tuning bits
//				  ..011111     +31	+12.5%
//				  ..000001     +1	+0.4%
//				  ..000000     0
//				  ..111111     -1	-0.4%
//				  ..100000     -32	-12.5%
	OSCTUNE = 0;
//
//		*** System option ***
//
//				  OPTION_REG - Option register
//				  11111111  Initial
//				  X.......  RBPU: PORTB Pull-up enable
//				  1.......     disabled
//				  0.......     enable
//				  .X......  INTEDG: Interrupt edge select
//				  .1......     riging edge
//				  .0......     failling edge
//				  ..X.....  T0CS: TMR0 clock source select
//				  ..1.....     Transition on GP2/T0CKI pin
//				  ..0.....     Internal instruction cycle
//				  ...X....  T0SE: TMR0 source edge select
//				  ...1....     high-to-low edge
//				  ...0....     low-to-high edge
//				  ....X...  PSA: Prescaler assignment
//				  ....1...     WDT
//				  ....0...     Timer0
//				  .....XXX  PS:  Prescaler assignment
//				  .....000     TMR0 rate 1:2 / WDT rate 1:1
//				  .....001               1:4 /          1:2
//				  .....010               1:8 /          1:4
//				  .....011              1:16 /          1:8
//				  .....100              1:32 /         1:16
//				  .....101              1:64 /         1:32
//				  .....110             1:128 /         1:64
//				  .....111             1:256 /        1:128
	OPTION =    0b11010100;		// Timer0=CLKO(4MHz/4)/32 -> 31.25KHz
//
//		*** Watchdock timer ***
//
//				  WDTCON - Watchdog control register
//				  ---0100?  Initial
//				  ...XXXX.  WDTPS: Watchdog timer period select bits
//				  ...0000.     1:32
//				  ...0001.     1:64
//				  ...0010.     1:128
//				  ...0011.     1:256
//				  ...0100.     1:512
//				  ...0101.     1:1024
//				  ...0110.     1:2048
//				  ...0111.     1:4096
//				  ...1000.     1:8192
//				  ...1001.     1:16384
//				  ...1010.     1:32768
//				  ...1011.     1:65536
//				  .......X  WDTPS: Watchdog timer period select bits
//				  .......1     WDT is turned on
//				  .......0     WDT is turned off
	WDTCON =    0b00010100;		// <WDT disable>
//	WDTCON =    0b00010101;		// 32.25KHz / 32768 ~= 1sec  <enable>
//	WDTCON =    0b00010111;		// 32.25KHz / 65536 ~= 2sec  <enable>
//
//		*** A/D Converter ***
//
//				  ADCON0 - A/D control register 0
//				  00000000  Initial
//				  XX......  ADCS: A/D converter clock select
//				  00......    ADCS2=0 Fosc/2   ADCS2=1 Fosc/4
//				  01......            Fosc/8           Fosc/16
//				  10......            Fosc/32          Fosc/64
//				  11......            FRC              FRC
//				  ..XXX...  CHS: Analog channel select bits
//				  ..000...    Channel 0 RA0/AN0
//				  ..001...    Channel 1 RA1/AN1
//				  ..010...    Channel 2 RA2/AN2
//				  ..011...    Channel 3 RA3/AN3
//				  ..100...    Channel 4 RA4/AN4
//				  ..101...    Channel 5 RB6/AN5
//				  ..110...    Channel 6 RB7/AN6
//				  .....X..  GO/DONE: A/D conversion status bit
//				  .....1..    ADON=1 in progress
//				  .....0..           not in progress
//				  .......X  ADON: A/D on bit
//				  .......1    module is operating
//				  .......0    module is shutoff
//	ADCON0 =    0b10000001;		// Fosc/2 <Channel dynamic change>
//	ADCON0 =    0b01000001;		// Fosc/8 <Channel dynamic change>
	ADCON0 =    0b11000001;		// FRC
//
//				  ADCON1 - A/D control register 1
//				  00000000  Initial
//				  X.......  ADFM: A/D result format select bit
//				  1.......    Right justified
//				  0.......    Left justified
//				  .X......  ADCS2: A/D clock divied by 2 select bit
//				  .1......    Enabled
//				  .0......    Disabled
//				  ..XX....  VCFG: A/D port configuration control bits
//				  ..00....    VREF+=AVdd  VREF-=AVss
//				  ..01....    VREF+=AVdd  VREF-=VREF-
//				  ..10....    VREF+=VREF+ VREF-=AVss
//				  ..11....    VREF+=VREF+ VREF-=VREF-
	ADCON1 =    0b10000000;		// Right adj, clock/1, Vref=Vdd:Vss
//	ADCON1 =    0b11000000;		// Right adj, clock/2, Vref=Vdd:Vss
//
//				  ANSEL - Analog select register
//				  01111111  Initial
//				  .X......  ANS6 1=Analog 0=Digital
//				  ..X.....  ANS5 1=Analog 0=Digital
//				  ...X....  ANS4 1=Analog 0=Digital
//				  ....X...  ANS3 1=Analog 0=Digital
//				  .....X..  ANS2 1=Analog 0=Digital
//				  ......X.  ANS1 1=Analog 0=Digital
//				  .......X  ANS0 1=Analog 0=Digital
	ANSEL =     0b00011111;		// 
//
//		*** GPIO port ***
//
//				  PORTA - General purpose I/O register A
//				  xxx00000  Initial
//				  X.......  RA7: OSC1/CLKI
//				  .X......  RA6: OSC2/CLKO
//				  ..X.....  RA5: MCLR/Vpp
//				  ...X....  RA4: AN4/T0CKI
//				  ....X...  RA3: AN3/Vref+
//				  .....X..  RA2: AN2/Vref-
//				  ......X.  RA1: AN1
//				  .......X  RA0: AN0
	PORTA =		0b11111111;		//
//
//				  PORTB - General purpose I/O register B
//				  xxxxxxxx  Initial
//				  X.......  RB7: AN6/T1OSI/PGD
//				  .X......  RB6: AN5/T1OSO/T1CKI/PGC
//				  ..X.....  RB5: SS/TX/CK
//				  ...X....  RB4: SCK/SCL
//				  ....X...  RB3: CCP1/PGM
//				  .....X..  RB2: SDO/RX/DT
//				  ......X.  RB1: SDI/SDA
//				  .......X  RB0: INT/CCP
	PORTB =		0b11111111;		//
//
//				  TRISA - GPIO tri-state register A
//				  11111111  Initial
//				  1.......  TRISA7: RA7 1=input 0=output
//				  .0......  TRISA6: RA6 1=input 0=output
//				  ..1.....  TRISA5: RA5 1=input only
//				  ...X....  TRISA4: RA4 1=input 0=output
//				  ....X...  TRISA3: RA3 1=input 0=output
//				  .....X..  TRISA2: RA2 1=input 0=output
//				  ......X.  TRISA1: RA1 1=input 0=output
//				  .......X  TRISA0: RA0 1=input 0=output
	TRISA =		0b10111111;		//
//
//				  TRISB - GPIO tri-state register B
//                 11111111  Initial
//				  X.......  TRISB7: RB7 1=input 0=output
//				  .X......  TRISB6: RB6 1=input 0=output
//				  ..X.....  TRISB5: RB5 1=input 0=output
//				  ...X....  TRISB4: RB4 1=input 0=output
//				  ....X...  TRISB3: RB3 1=input 0=output
//				  .....X..  TRISB2: RB2 1=input 0=output
//				  ......X.  TRISB1: RB1 1=input 0=output
//				  .......X  TRISB0: RB0 1=input 0=output
	TRISB =		0b00000000;		//

//for(;;);
//		*** LCD module ***
	LCD_Init();
	LCD_Start();
	printf("test3");

	for(;;) {
		short a;

#define AD_VREF			5000			// mV
#define AD_FULLSCALE	1024			// Count
#define AD_AUTORANGE	700				// Count 3.5V
#define V1_CH			0				// A/D ANx
#define V2_CH			1				// A/D ANx
#define I1_CH			3				// A/D ANx
#define I2_CH			4				// A/D ANx
#define I3_CH			2				// A/D ANx
#define V1_GAIN_A		1				// Terminal->Pin A/B
#define V2_GAIN_A		1				// Terminal->Pin A/B
#define I1_GAIN_A		1001			// Terminal->Pin A/B
#define I2_GAIN_A		101				// Terminal->Pin A/B
#define I3_GAIN_A		1				// Terminal->Pin A/B
#define V1_GAIN_B		1				// Terminal->Pin A/B
#define V2_GAIN_B		11				// Terminal->Pin A/B
//#define R_SENSE		0.1				// ohm Current->Voltage convert register
//#define I1_GAIN_B		(1/R_SENSE*2)	// Terminal->Pin A/B x2=AntiMinusOffsetMixer
//#define I2_GAIN_B		(1/R_SENSE*2)	// Terminal->Pin A/B x2=AntiMinusOffsetMixer
//#define I3_GAIN_B		(1/R_SENSE*2)	// Terminal->Pin A/B x2=AntiMinusOffsetMixer
#define I1_GAIN_B		20				// Terminal->Pin A/B x2=AntiMinusOffsetMixer
#define I2_GAIN_B		20				// Terminal->Pin A/B x2=AntiMinusOffsetMixer
#define I3_GAIN_B		20				// Terminal->Pin A/B x2=AntiMinusOffsetMixer

		// Voltage
		if(1) {
			short	v;

			a = getads(V1_CH, 20);
			if (a < AD_AUTORANGE) {
				v = (long)a * AD_VREF * V1_GAIN_B / AD_FULLSCALE / V1_GAIN_A;
				LCD_Position(0,0);
				printf("%d.%02dV ", v/1000, ((v%1000+5)/10));
			} else {
				a = getads(V2_CH, 20);
				v = (long)a * AD_VREF * V2_GAIN_B / AD_FULLSCALE / V2_GAIN_A;
				LCD_Position(0,0);
				printf("%d.%dV ", v/1000, ((v%1000+50)/100));
			}
		}

		// Current
		if(1) {
			short	i;

			a = getads(I1_CH, 20);
			if (a < AD_AUTORANGE) {
				i = (long)(a - ofs_i1) * AD_VREF * I1_GAIN_B * 10 / AD_FULLSCALE / I1_GAIN_A;
				LCD_Position(1,0);
				printf("%d.%dmA ", i/10, i%10);
			} else {
				a = getads(I2_CH, 20);
				if (a < AD_AUTORANGE) {
					i = (long)(a - ofs_i2) * AD_VREF * I2_GAIN_B / AD_FULLSCALE / I2_GAIN_A;
					LCD_Position(1,0);
					printf("%dmA ", i);
				} else {
					a = getads(I3_CH, 20);
					i = (long)(a - ofs_i3) * AD_VREF * I3_GAIN_B / AD_FULLSCALE / I3_GAIN_A;
					LCD_Position(1,0);
					printf("%3d.%03dA ", i/1000, i%1000);
				}
			}
		}

		__delay_ms(100);
		if (!RA7) {
			if (!sw_a) {
				ofs_i1 = getads(I1_CH, 20);
				ofs_i2 = getads(I2_CH, 20);
				ofs_i3 = getads(I3_CH, 20);
				beep();
			}
			sw_a = 1;
		} else {
			sw_a = 0;
		}
		if (!RA6) {
			click();
		}
	}
}